<?php
require_once 'system/setup.conf';
require_once 'warp/warplib/applet.inc';
require_once 'warp/warplib/appletutils.inc';


define("INFORMATION_MASK_FILESIZE", 1);
define("INFORMATION_MASK_PERMISSIONS", 2);
define("INFORMATION_MASK_FILETYPE", 4);


class MarkedFile
{
  var $_url;   // as accessible from the web server
  var $_path;  // including filename
  var $_title; // the text link (or information) to display - usually filename
  var $_description; // additional information - e.g. filesize
  var $_idx;
  var $_dpage;
  var $_fpage;
  var $_dpageSize;
  var $_fpageSize;

  function MarkedFile($idx, $url, $path, $title, $description) {
     $this->_idx = $idx;
     $this->_url = $url;
     $this->_path = $path;
     $this->_title = $title;
     $this->_description = $description;
  }

  function getFileTitle() {
     return basename($this->_title);
  }

  function getFileName() {
     return basename($this->_url);
  }

  function getFilePathSystem() {
     return $this->_path;
  }

  function getFilePathWebServer() {
     return $this->_url;
  }

  function getRenderLine() {
     $line = "";

     if ($this->_url) {
         $line .= "<a href='".$this->_url."'>$this->_title</a>";
     } else {
         $line .= $this->_title;
     }
 
     $line .= ($this->_description ? " " . $this->_description : "");

     return $line;
  }

}

class MarkedDirectory
{
  var $_dir;
  var $_title;

  function MarkedDirectory($dir, $title) {
     $this->_dir = $dir;
     $this->_title = $title;
  }

  function getRenderLine($appMan, $app) {
     return $appMan->getAppletLink($app, "dir", $this->_dir, $this->_title);
  }

}


/*
The root path is where, in the filesystem, we are allowed to start browsing from.
The mappedpath (when present) is where, in the web server, is equivalent to the root path.
  Mappedpath needn't exist (since you might want to keep the contents of files under rootpath
  secret), but can be included when you want easy access to some contents.
*/
class Warp_Browser_Applet extends Warp_Applet
{
    var $_dir;
    var $_dirlist;
    var $_filelist;
    var $_rootpath;
    var $_mappedpath;
    var $_sortFilenames;
    var $_sortDirnames;
    var $_information;

    function Warp_Browser_Applet($caption = "Browser", $rootpath = NULL, $mappedpath = NULL)
    {
        parent::__construct($caption);
        if ($rootpath == NULL || $rootpath == "") {
           $this->_rootpath = $this->getConfFileName($this->_dir);
        } else {
           # Absolute paths are used as is,
           # relative paths are treated relative to our root
           if (substr($rootpath, 0, 1) == "/") {
              $this->_rootpath = $rootpath;
           } else {
              $this->_rootpath = WarpSetup::getPathRoot()."/".$rootpath;
           }
        }
        $this->_rootpathRelative = $rootpath;
        $this->_mappedpath = $mappedpath;
        $this->_dpageSize = 20;
        $this->_fpageSize = 15;
        $this->_sortFilenames = $this->_sortDirnames = true;
        $this->_information = 0;
    }

    function getDirectory()
    {
        return "browser";
    }

    function init(&$appMan)
    {
        parent::init($appMan);

        $this->_dpage = $appMan->queryParameter($this, "dpset");
        $this->_fpage = $appMan->queryParameter($this, "fpset");

        if ($this->_dpage == "") {
           $this->_dpage = $appMan->queryParameter($this, "dpage");
        }
        if ($this->_fpage == "") {
           $this->_fpage = $appMan->queryParameter($this, "fpage");
        }

        if ($this->_dpage == "") $this->_dpage = 0;
        if ($this->_fpage == "") $this->_fpage = 0;

        $dir = $appMan->queryParameter($this, "dir");
        $curr = $appMan->queryParameter($this, "current");

        if ($dir != "") {
           $this->_dir = $dir;
           // If we specify a directory, reset the page counters to zero
           $this->_dpage = 0;
           $this->_fpage = 0;
        } else { 
           $this->_dir = $curr;
        }
        if ($this->_dir == "") {
           $this->_dir = "/";
        }

        $this->_dir = appletUtils::validatePath($this->_dir);
        $this->parseDirectory($appMan, $this->_dir);

        return true;
    }

    function showFileSizes() 
    {
        $this->_information |= INFORMATION_MASK_FILESIZE;
    }

    function showFileTypes() 
    {
        $this->_information |= INFORMATION_MASK_FILETYPE;
    }

    function showFilePermissions() 
    {
        $this->_information |= INFORMATION_MASK_PERMISSIONS;
    }

    function renderPanel(&$appMan)
    {
        $html = '';
        return $html;
    }

    function getDirectoryPane($appMan)
    {
        $dirpane = '';

        $parentDir = appletUtils::getParentDirectory($this->_dir);
        if ($parentDir != $this->_dir) {
           $dirpane = $this->renderParentDirectoryLine($appMan, $parentDir);
        }

        $size = count($this->_dirlist);
        $first = $this->_dpage * $this->_dpageSize;
        $last = $first + $this->_dpageSize;
        $first = max($first, 0);
        $last = min($last, $size);

        for($i=$first;$i<$last;++$i) {
           $name = $this->_dirlist[$i];
           $dirpane .= $this->renderDirectoryLine($appMan, $name);
        }

        $dirpane .= appletUtils::getPagerSelect($appMan, $this, $this->_dpage, $size, $this->_dpageSize, "dpset");

        return $dirpane;
    }

    function getFilePane(&$appMan)
    {
        $filepane = "";

        $size = count($this->_filelist);
        $first = $this->_fpage * $this->_fpageSize;
        $last = $first + $this->_fpageSize;
        $first = max($first, 0);
        $last = min($last, $size);

        for($i=$first;$i<$last;++$i) {
           $name = $this->_filelist[$i];
           $filepane .= $this->renderFileLine($appMan, $name);
        }

        $filepane .= appletUtils::getPagerSelect($appMan, $this, $this->_fpage, $size, $this->_fpageSize, "fpset");

        return $filepane;
    }
  
    function getDirectoryBrowser(&$appMan, $dir)
    {
        $dirs = split("/", $dir);
        $top = $appMan->getAppletLink($this, "dir", "/", "[Root]");
        $path = "";
        foreach($dirs AS $d) {
           if ($d == "") continue;
           $path.= "/$d";
           $top .= "&nbsp;/&nbsp;";
           $top .= $appMan->getAppletLink($this, "dir", $path, "$d");
        }
        $top .= "<br/><br/>";
        return $top;
    }

    function renderWindow(&$appMan)
    {
        $top = $this->getDirectoryBrowser($appMan, $this->_dir);
        $filepane = $this->getFilePane($appMan);
        $dirpane = $this->getDirectoryPane($appMan);
        return $top . $appMan->getView()->split2V($dirpane, $filepane);
    }

    function renderInfo(&$appMan)
    {
       return "This is the generic browser applet, it is not usually used on its own, but rather as a base class. See MP3Play and Explorer for examples. The root directory in each case should be specified when you construct (via new) the applet.";
    }

    function renderFileLine(&$appMan, $f)
    {
       return $f->getRenderLine()."<br/>";
    }

    function renderDirectoryLine($appMan, $d)
    {
       return $d->getRenderLine($appMan, $this)."<br/>";
    }

    function renderParentDirectoryLine($appMan, $p) 
    {
       return $appMan->getAppletLink($this, "dir", $p, "[Parent]")."<br/>";
    }

    function getRefreshParams(&$appMan)
    {
        return 
          $appMan->getArgument($this, "current", $this->_dir)."&".
          $appMan->getArgument($this, "dpage", $this->_dpage)."&".
          $appMan->getArgument($this, "fpage", $this->_fpage);
    }

//
	
	function sortFilename($a, $b) {
		return $a->_url > $b->_url;
	}

	function sortDirectory($a, $b) {
		return $a->_dir > $b->_dir;
	}

    function parseDirectory(&$appMan, $dir) {

       $fullPath = $this->_rootpath."/".$dir;

       $this->_dirlist = array();
       $this->_filelist = array();

       $this->getDirectoryList($appMan, $dir, $this->_dirlist, $this->_filelist);
	}

    function getDirectoryList(&$appMan, $dir, &$dirlist, &$filelist) {
       $fullPath = $this->_rootpath."/".$dir;

       if (@$handle = opendir($fullPath)) {
			 $fileList = array();
          while (false !== ($file = readdir($handle))) {
              if (substr($file, 0, 1) != ".") {
                  if (is_dir("$fullPath/$file")) {
                     if (is_file("$fullPath/$file/.name")) {
                        $name = file_get_contents("$fullPath/$file/.name");
                     } else {
                        $name = $file;
                     }

                    $file = appletUtils::validatePath("$dir/$file");
                    $name = appletUtils::validatePath($name);
                    $dirlist[] = new MarkedDirectory($file, $name);

                  } else if (is_file("$fullPath/$file")) {
							$fileList[] = $file;
                  }
             }
          }
          closedir($handle);

			if ($this->_sortFilenames) {
				sort($fileList);//, array("Warp_Browser_Applet","sortFilename"));
			}
			$fileIdx = 0;
			foreach($fileList as $file) {
            $nativeFilename = "$fullPath/$file";

            // The URL link for the file
            $url = "";
            $webpath = getFileSystemRoot() . $this->_rootpath . "$dir/$file";
            //$webpath = $this->_rootpath . "$dir/$file";

            // If we've given an explicit path, use it.
            if ($this->_mappedpath) {
                $url = $this->_mappedpath . "/$dir/$file";

            } else if (file_exists($webpath)) {
                // If this file is accessible from the web server path then
                // use the relative URL, but force it to server root.
                $url = "/" . $this->_rootpathRelative . "$dir/$file";
            }

            // Any supplementary information
            $info = "";
            if ($this->_information & INFORMATION_MASK_FILESIZE) {
               $info .= "(" . filesize($nativeFilename) . ") ";
            }

            if ($this->_information & INFORMATION_MASK_FILETYPE) {
               $info .= "(" . filetype($nativeFilename) . ") ";
            }

            if ($this->_information & INFORMATION_MASK_PERMISSIONS) {
               $info .= "(" . substr(sprintf('%o', fileperms($nativeFilename)), -4) . ") ";
            }
 
            // Then create the entry
            if ($this->includeFile($nativeFilename)) {
				    $filelist[] = new MarkedFile($fileIdx, $url, $nativeFilename, $file, $info);
				    ++$fileIdx;
            }
			}

			if ($this->_sortDirnames) {
				usort($dirlist, array("Warp_Browser_Applet","sortDirectory"));
			}
      }
  }

    function includeFile($fn) {
        return true;
    }

}

?>


